//+------------------------------------------------------------------+
//|                                                       HMA EA.mq5 |
//|                        Copyright 2019, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2019, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"
#include<Trade\Trade.mqh>

CTrade  trade;

input string      hmaP  ="HMA Parameters";
input int                inpPeriod  = 20;          // Period
input double             inpDivisor = 2.0;         // Divisor ("speed")
input ENUM_APPLIED_PRICE inpPrice   = PRICE_CLOSE; // Price

input string               TradeP      = "Trade Parameters";
input double               LotSize     = 0.1;
input bool                 AutoLot     = true;     // Use Money Management
input double               RiskPercent = 0.1;
input double               LotStep     = 1;
input double               StopLoss    = 0.1;
double               TakeProfit  = 0;
input int                  MagicNumber = 45343;

input bool     UseTimeFilter  = true;
input bool     TradeMonday    = true;
input string   StartHour      = "03:00"; // Monday Start Hour
input string   EndHour        = "17:59"; // Monday End Hour
input bool     TradeTuesday    = true;
input string   StartHour2     = "03:00"; // Tuesday Start Hour
input string   EndHour2       = "17:59"; // Tuesday End Hour
input bool     TradeWednesday = true;
input string   StartHour3     = "03:00"; // Wednesday Start Hour
input string   EndHour3       = "17:59"; // Wednesday End Hour
input bool     TradeThursday  = true;
input string   StartHour4     = "03:00"; // Thursday Start Hour
input string   EndHour4       = "17:59"; // Thursday End Hour
input bool     TradeFriday    = true;
input string   StartHour5     = "03:00"; // Friday Start Hour
input string   EndHour5       = "17:59"; // Friday End Hour

double   MinLot         = SymbolInfoDouble(Symbol(),SYMBOL_VOLUME_MIN);
double   MaxLot         = SymbolInfoDouble(Symbol(),SYMBOL_VOLUME_MAX);

double Ask, Bid, bt[], st[];
double Open[], High[], Low[], Close[];
int tradenow, stHandle, lastEntry;
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//---
   //--- set MagicNumber for your orders identification
   trade.SetExpertMagicNumber(MagicNumber);
   
   //--- set available slippage in Points() when buying/selling
   int deviation=50;
   trade.SetDeviationInPoints(deviation);
   
   //--- logging mode: it would be better not to declare this method at all, the class will set the best mode on its own
   trade.LogLevel(1); 
   
   
   //--- what function is to be used for trading: true - OrderSendAsync(), false - OrderSend()
   trade.SetAsyncMode(false);
   
   stHandle=iCustom(Symbol(),0,"Hull average 2",inpPeriod,inpDivisor,inpPrice);
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---
   
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---
   Ask = SymbolInfoDouble(Symbol(),SYMBOL_ASK);
   Bid = SymbolInfoDouble(Symbol(),SYMBOL_BID);
   if(IsNewCandle())tradenow=1;
   
   ArraySetAsSeries(st,true);
   CopyBuffer(stHandle,1,0,3,st);
   
   int count=50;   // number of elements to copy
   //ArraySetAsSeries(Time,true);
   //CopyTime(_Symbol,_Period,0,count,Time);
   ArraySetAsSeries(Low,true);
   CopyLow(_Symbol,_Period,0,count,Low);
   ArraySetAsSeries(High,true);
   CopyHigh(_Symbol,_Period,0,count,High);
   ArraySetAsSeries(Open,true);
   CopyOpen(_Symbol,_Period,0,count,Open);
   ArraySetAsSeries(Close,true);
   CopyClose(_Symbol,_Period,0,count,Close);
   
   if(st[2]==2 && st[1]==1){
      CLOSEALL(1);
      OrderEntry(0);
   }
   if(st[2]==1 && st[1]==2){
      CLOSEALL(0);
      OrderEntry(1);
   }
  }
//+------------------------------------------------------------------+

double lastclose; 
//+------------------------------------------------------------------+
//insuring its a new candle function
//+------------------------------------------------------------------+
bool IsNewCandle()
{
   if(iClose(Symbol(),PERIOD_CURRENT,1)==lastclose){return(false);}
   if(iClose(Symbol(),PERIOD_CURRENT,1)!=lastclose){lastclose=iClose(Symbol(),PERIOD_CURRENT,1);return(true);}
	return(true);
}

//+------------------------------------------------------------------+
double bsl, btp, ssl, stp;
void OrderEntry(int direction){
   double lotsize=LotSize;
   if(direction==0 && OpenOrdersThisPair(Symbol())<1 && tradenow==1 && (lastEntry==0||lastEntry==2) && (!UseTimeFilter||WorkingHour())){
      
      if(StopLoss>0){bsl=NormalizeDouble(Ask-((StopLoss/100)*Ask),Digits());}else{bsl=0;}
      if(TakeProfit>0){btp=Ask+TakeProfit*Point();}else{btp=0;}
      
      if(AutoLot)lotsize=MMComputeRisk(RiskPercent,Ask-bsl);
      trade.Buy(lotsize,NULL,Ask,bsl,btp,"EA Trade");
      tradenow=0;
      lastEntry=1;
   }
   
   if(direction==1  && OpenOrdersThisPair(Symbol())<1 && tradenow==1 && (lastEntry==0||lastEntry==1) && (!UseTimeFilter||WorkingHour())){
      
      if(StopLoss>0){ssl=NormalizeDouble(Bid+((StopLoss/100)*Bid),Digits());}else{ssl=0;}
      if(TakeProfit>0){stp=Bid-TakeProfit*Point();}else{stp=0;}
      
      if(AutoLot)lotsize=MMComputeRisk(RiskPercent,ssl-Bid);
      trade.Sell(lotsize,NULL,Bid,ssl,stp,"EA Trade");
      tradenow=0;
      lastEntry=2;
   } 
}

//+-------------------------------
//+ Check Open Orders
//+-------------------------------

int OpenOrdersThisPair(string pair)
{
  int tnum =0;
   int _tp=PositionsTotal();
   
   for(int i=_tp-1; i>=0; i--)
     {
      string _p_symbol=PositionGetSymbol(i);
      ulong tick = PositionGetTicket(i);
      
      if(PositionSelectByTicket(tick)){
         int posMag = PositionGetInteger(POSITION_MAGIC);
      
      if(MagicNumber==posMag && _p_symbol==pair){
         tnum++;
         }
    }
    //}
    }
return(tnum);
}

void CLOSEALL(int type){

   int _tp=PositionsTotal();
   for(int i=_tp-1; i>=0; i--)
     {
      string _p_symbol=PositionGetSymbol(i);
      ulong tick=PositionGetTicket(i);
      
      
      if(PositionSelectByTicket(tick)){
      int typer=(int)PositionGetInteger(POSITION_TYPE);
      int posmag=(int)PositionGetInteger(POSITION_MAGIC);
      
      if(MagicNumber==posmag && typer==type && _p_symbol==Symbol()){
           
      trade.PositionClose(tick,-1);
             

         }
      }
   }
}


double MMComputeRisk(double percent, double sl)
{

   double MMBalance, MMRiskMoney;
   double MMMaxLot=MaxLot;
   double MMMinLot=MinLot;
   double MMLotStep= LotStep; 
   double MMTickValue = SymbolInfoDouble( _Symbol, SYMBOL_TRADE_TICK_VALUE);
   double MMTickSize = SymbolInfoDouble( _Symbol, SYMBOL_TRADE_TICK_SIZE);
   
   int lotdigits=0;
   do
   {
      lotdigits++;
      MMLotStep*=10;
   }while(MMLotStep<1);
   if(LotStep==1)lotdigits=0;
   
   MMBalance = MathMin(AccountInfoDouble(ACCOUNT_BALANCE), AccountInfoDouble(ACCOUNT_EQUITY));
   MMRiskMoney=MMBalance*(RiskPercent/100);
   double lot = MMRiskMoney / ( sl * ( MMTickValue / MMTickSize ) + 1 );
   lot=NormalizeDouble( MathFloor(lot*MathPow(10,lotdigits))/MathPow(10,lotdigits), lotdigits );
   //Print(lot);
   if( lot > MMMaxLot ){ Comment("Trade Volume too Large for Asset: ",DoubleToString(lot,2)); lot=0; }
   if( lot < MMMinLot ){ Comment("Trade Volume too Small for Asset: ",DoubleToString(lot,2)); lot=0; }

   return NormalizeDouble( lot, lotdigits );
   
}

bool WorkingHour() {
  datetime datetime_0 = TimeCurrent();

  if(TradeMonday && TimeDayOfWeek(TimeCurrent())==1){ 
   datetime gi_time_01 = StringToTime(TimeToString(TimeCurrent(), TIME_DATE) + " " + StartHour);
   datetime gi_time_02 = StringToTime(TimeToString(TimeCurrent(), TIME_DATE) + " " + EndHour);
  
   if( gi_time_01 < gi_time_02 && gi_time_01 <= datetime_0 && datetime_0 <= gi_time_02 ) return (true);
   if( gi_time_01 > gi_time_02 && (datetime_0 >= gi_time_01 || datetime_0 <= gi_time_02) ) return (true);
  }
  if(TradeTuesday && TimeDayOfWeek(TimeCurrent())==2){ 
     datetime gi_time_01 = StringToTime(TimeToString(TimeCurrent(), TIME_DATE) + " " + StartHour2);
     datetime gi_time_02 = StringToTime(TimeToString(TimeCurrent(), TIME_DATE) + " " + EndHour2);
     
   if( gi_time_01 < gi_time_02 && gi_time_01 <= datetime_0 && datetime_0 <= gi_time_02 ) return (true);
   if( gi_time_01 > gi_time_02 && (datetime_0 >= gi_time_01 || datetime_0 <= gi_time_02) ) return (true);
  }
  if(TradeWednesday && TimeDayOfWeek(TimeCurrent())==3){ 
   datetime gi_time_01 = StringToTime(TimeToString(TimeCurrent(), TIME_DATE) + " " + StartHour3);
   datetime gi_time_02 = StringToTime(TimeToString(TimeCurrent(), TIME_DATE) + " " + EndHour3);
   
   if( gi_time_01 < gi_time_02 && gi_time_01 <= datetime_0 && datetime_0 <= gi_time_02 ) return (true);
   if( gi_time_01 > gi_time_02 && (datetime_0 >= gi_time_01 || datetime_0 <= gi_time_02) ) return (true);
  }
  if(TradeThursday && TimeDayOfWeek(TimeCurrent())==4){ 
   datetime gi_time_01 = StringToTime(TimeToString(TimeCurrent(), TIME_DATE) + " " + StartHour4);
   datetime gi_time_02 = StringToTime(TimeToString(TimeCurrent(), TIME_DATE) + " " + EndHour4);
  
   if( gi_time_01 < gi_time_02 && gi_time_01 <= datetime_0 && datetime_0 <= gi_time_02 ) return (true);
   if( gi_time_01 > gi_time_02 && (datetime_0 >= gi_time_01 || datetime_0 <= gi_time_02) ) return (true);
  }
  if(TradeFriday && TimeDayOfWeek(TimeCurrent())==5){ 
   datetime gi_time_01 = StringToTime(TimeToString(TimeCurrent(), TIME_DATE) + " " + StartHour5);
   datetime gi_time_02 = StringToTime(TimeToString(TimeCurrent(), TIME_DATE) + " " + EndHour5);
  
   if( gi_time_01 < gi_time_02 && gi_time_01 <= datetime_0 && datetime_0 <= gi_time_02 ) return (true);
   if( gi_time_01 > gi_time_02 && (datetime_0 >= gi_time_01 || datetime_0 <= gi_time_02) ) return (true);
  }
  
   return(false);
}

int TimeDayOfWeek(datetime date)
  {
   MqlDateTime tm;
   TimeToStruct(date,tm);
   return(tm.day_of_week);
  }